#include "cardappz.h"
#include <assert.h>
#include <stdio.h>

unsigned char PURSE_MASTER_KEY0[16] =
  { 0x00, 0x00, 0x0B, 0x00, 0x00, 0x50, 0x41, 0x02, 0x00, 0x00, 0x1B, 0x00,
  0x00, 0x50, 0x41, 0x02
};
unsigned char PURSE_MASTER_KEY1[16] =
  { 0x00, 0x00, 0x0A, 0x00, 0x00, 0x50, 0x41, 0x02, 0x00, 0x00, 0x1A, 0x00,
  0x00, 0x50, 0x41, 0x02
};

int DisplayPurse(CARDAPPZ_CTX_ST * ctx)
{
  PURSE_RELOAD_ST ReloadEvent;
  PURSE_PURCHASE_ST PurchaseEvent;
  int ReloadEvents = 0, PurchaseEvents = 0;
  unsigned long Balance;
  int Index;
  int rc;

  if (ctx == NULL)
    return CARDAPPZ_INVALID_CONTEXT;

  /* Select the purse application */
  printf("Entering purse application\n");
  rc = SelectPurse(ctx);
  if (rc)
    goto done;

  /* Read balance */
  printf("Reading Balance...\n");
  rc = ReadPurseBalance(ctx, &Balance);
  if (rc)
  {
    printf("Balance not found\n");
    goto done;
  }
  printf("Current Balance is %ld\n", Balance);

  /* Read re-load events */
  for (Index = 0;; Index++)
  {
    printf("Reading Re-load Event %d...\n", Index + 1);
    rc = ReadPurseReloadEvent(ctx, Index, &ReloadEvent);
    if (rc)
      break;
    ReloadEvents++;
    printf("Content of Re-load Event %d :\n", Index + 1);
    // TODO
  }

  if (!ReloadEvents)
    printf("No Re-load Event in the card, it has never been loaded\n");

  /* Read purchase events */
  for (Index = 0;; Index++)
  {
    printf("Reading Purchase Event %d...\n", Index + 1);
    rc = ReadPursePurchaseEvent(ctx, Index, &PurchaseEvent);
    if (rc)
      break;
    PurchaseEvents++;
    printf("Content of Purchase Event %d :\n", Index + 1);
    // TODO
  }

  if (!PurchaseEvents)
    printf("No Purchase Event in the card, it has never been used\n");

  rc = CARDAPPZ_OK;

done:
  if (rc)
    printf("Error %d in DisplayPurse\n", rc);
  return rc;
}

int SelectPurse(CARDAPPZ_CTX_ST * ctx)
{
  assert(ctx != NULL);
  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      return DesfireSelectPurse(ctx);
  }
  return CARDAPPZ_NOT_YET_IMPLEMENTED;
}

int ReadPurseBalance(CARDAPPZ_CTX_ST * ctx, unsigned long *Balance)
{
  assert(ctx != NULL);
  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      return DesfireReadPurseBalance(ctx, Balance);
  }
  return CARDAPPZ_NOT_YET_IMPLEMENTED;
}

int ReadPurseReloadEvent(CARDAPPZ_CTX_ST * ctx, int Index,
                         PURSE_RELOAD_ST * ReloadEntry)
{
  int rc;

  assert(ctx != NULL);

  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      rc = DesfireReadPurseReloadEvent(ctx, Index, ReloadEntry);
      break;

    default:
      rc = CARDAPPZ_NOT_YET_IMPLEMENTED;
      break;
  }

  /* Convert to computer endianess */
  if (ReloadEntry != NULL)
  {
    ReloadEntry->Credit = CtoHs(ReloadEntry->Credit);
    ReloadEntry->NewBalance = CtoHl(ReloadEntry->NewBalance);
  }

  return rc;
}

int ReadPursePurchaseEvent(CARDAPPZ_CTX_ST * ctx, int Index,
                           PURSE_PURCHASE_ST * PurchaseEntry)
{
  int rc;

  assert(ctx != NULL);

  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      rc = DesfireReadPursePurchaseEvent(ctx, Index, PurchaseEntry);
      break;

    default:
      rc = CARDAPPZ_NOT_YET_IMPLEMENTED;
      break;
  }

  /* Convert to computer endianess */
  if (PurchaseEntry != NULL)
  {
    PurchaseEntry->Debit = CtoHs(PurchaseEntry->Debit);
  }

  return rc;
}

int DebitPurse(CARDAPPZ_CTX_ST * ctx, PURSE_PURCHASE_ST * PurchaseEntry)
{
  int rc;

  assert(ctx != NULL);

  /* Ensure appropriate endianess on card */
  if (PurchaseEntry != NULL)
  {
    PurchaseEntry->Debit = HtoCs(PurchaseEntry->Debit);
  }

  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      rc = DesfireDebitPurse(ctx, PurchaseEntry);
      break;

    default:
      rc = CARDAPPZ_NOT_YET_IMPLEMENTED;
      break;
  }

  /* Back to computer endianess */
  if (PurchaseEntry != NULL)
  {
    PurchaseEntry->Debit = CtoHs(PurchaseEntry->Debit);
  }

  return rc;
}

int CreditPurse(CARDAPPZ_CTX_ST * ctx, PURSE_RELOAD_ST * ReloadEntry)
{
  int rc;

  assert(ctx != NULL);

  /* Ensure appropriate endianess on card */
  if (ReloadEntry != NULL)
  {
    ReloadEntry->Credit = HtoCs(ReloadEntry->Credit);
    ReloadEntry->NewBalance = HtoCl(ReloadEntry->NewBalance);
  }

  switch (ctx->card_type)
  {
    case CARD_DESFIRE:
      rc = DesfireCreditPurse(ctx, ReloadEntry);
      break;

    default:
      rc = CARDAPPZ_NOT_YET_IMPLEMENTED;
      break;
  }

  /* Back to computer endianess */
  if (ReloadEntry != NULL)
  {
    ReloadEntry->Credit = CtoHs(ReloadEntry->Credit);
    ReloadEntry->NewBalance = CtoHl(ReloadEntry->NewBalance);
  }

  return rc;
}
